function module (config, errback) {
   "use strict";

   var exports = {},
       _ = require('underscore'),
       async = require('async'),
       timer,
       writeFile = require('fs').writeFile,
       rename = require('fs').rename,
       log = config.log,
       statusData = {},
       food = require('../models/food').client();

   function status (callback) {
      callback(null, statusData);
   }

   /* Function: getMenus
    * Grab the menus for each location and write the results to a file
    */

   function getMenus () {
      var locations = food.getLocations(),
          output = {},
          jobs = [],
          mealNames,
          processMenu;

      mealNames = ['Breakfast', 'Lunch', 'Dinner', 'Knight%20Room', 'Late%20Knight'];

      _(locations).each(function (info, title) {
         _(mealNames).each(function (meal) {

            // add a job for each mealName for each location that grabs the
            // current menu and adds it to output
            jobs.push(function (callback) {
               food.getMenu(title, meal, function (err, data) {
                  if (err) return callback(err);

                  output[info.title] = output[info.title] || {};
                  output[info.title][meal] = data;

                  callback();
               });
            });
         });
      });

      async.parallel(jobs, function (err) {
         var ret = [],
             date = new Date().getTime();

         if (err) return log.error(err);
         log.info('food: got data, writing to file');

         try {
            _(output).each(function (meals, locName) {
               var retMeals = [];

               // make json that the app is able to parse
               _(meals).each(function (genres, locTitle) {
                  var retGenres = [];
                  _(genres).each(function (foods, genreName) {
                     retGenres.push({
                        genre_name: genreName,
                        items: foods
                     });
                  });

                  retMeals.push({
                     meal_name: locTitle.replace("%20", " "),
                     meal_avail: retGenres.length === 0? false : true,
                     genres: retGenres.length === 0? undefined : retGenres
                  });
               });

               // sort the meals so they appear in order
               var order = {
                  "Breakfast": 1,
                  "Lunch": 2,
                  "Dinner": 3,
                  "Knight Room": 4,
                  "Late Knight": 5
               };

               retMeals.sort(function (a, b) {
                  return order[a.meal_name] - order[b.meal_name];
               });

               ret.push({
                  location_name: locName,
                  date: date,
                  meals: retMeals
               });
            });

            ret.sort(function (a, b) {
               return a.location_name.localeCompare(b.location_name);
            });
         } catch (e) {
            log.error(e);
         }

         writeFile(
            config.paths.temp + '/rutgers-dining.txt',
            JSON.stringify(ret),
            function (err) {
               if (err) return log.error(err);

               rename(
                  config.paths.temp + '/rutgers-dining.txt',
                  config.paths.output + '/rutgers-dining.txt',
                  function (err) {
                     if (err) return log.error(err);

                     log.info('food: completed scrape');
                     statusData = {last: new Date()};
                  }
               );
            }
         );
      });
   }

   function start () {
      log.info('food: starting');
      timer = setInterval(getMenus, Number(config.modules.dining.interval) * 1000 * 60);
      getMenus();
   }

   function shutdown (done) {
      clearInterval(timer);
      done();
   }

   exports.start = start;
   exports.shutdown = shutdown;
   exports.status = status;

   return exports;
}

exports.module = module;
