exports.main = function (command, options) {
   var dnode      = require('dnode'),        // rpc framework
       Ax         = require('./ax'),
       log,
       yaml       = require('js-yaml'),
       _          = require('underscore'),
       version    = require('./revision').ver,
       async      = require('async'),
       handlers   = {},
       modules    = {},
       error      = false,
       config,
       rpc;

   require('winston-syslog').Syslog;

   handlers.status = function (done) {
      if (error) {
         done(error, null);
      } else {
         async.map(_.keys(modules), function (item, callback) {
            modules[item].status(callback);
         }, function (err, data) {
            done(null, data);
         });
      }
   };

   handlers.reseterror = function (done) {
      if (error) {
         error = false;
         done(null, 'removed error condition');
      } else {
         done('error condition is not set', null);
      }
      //TODO: winston
   };

   handlers.stop = function (done) {
      shutdown(function () {
         process.exit(0);
      });
   };

   function onError (name, error) {
      //TODO: winston
   }

   function clearRequireCache () {
      for (var i in require.cache) {
         if (require.cache.hasOwnProperty(i)) {
            delete require.cache[i];
         }
      }
   }

   function shutdown (done) {
      async.forEach(_.keys(modules), function (item, next) {
         modules[item].shutdown(next);
      }, function (err) {
         if (err) {
            log.error(err);
         } else {
            done();
         }
      });
      log.info('unloaded modules');
   }

   function initialize () {
      // TODO: fix paths
      log.info("RU Mobile API:" + version.ver + " " + version.rev + " starting.");
      clearRequireCache();
      config = require('./config.yaml').shift();
      config.log = log;
      _.each(config.modules, function (cfg, module) { 
         modules[module] = require('./modules/' + module).module(config, onError);
         modules[module].start();
      });
      log.info('loaded config and modules');
   }

   process.on('SIGHUP', function () {
      //TODO: winston
      log.info('received SIGHUP, reloading config and modules');
      shutdown(initialize);
   });

   process.on('SIGKILL', function () {
      log.info('received SIGKILL, shutting down');
      shutdown(function () {
         process.exit(0);
      });
   });

   process.on('uncaughtException', function (err) {
      log.error('Received uncaught exception: ' + require('util').inspect(err));
      log.error('reloading config and modules');
      shutdown(initialize);
   });

   config = require('./config.yaml').shift();

   if (options.debug) {
      log = new Ax();
   } else {
      log = new Ax(config.log);
   }

   initialize();

   process.title = "RU Mobile API:" + version.ver + " " + version.rev;

   rpc = dnode(handlers).listen(63200);
};

